package data.scripts;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.InteractionDialogAPI;
import com.fs.starfarer.api.campaign.InteractionDialogPlugin;
import com.fs.starfarer.api.campaign.OptionPanelAPI;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.TextPanelAPI;
import com.fs.starfarer.api.campaign.VisualPanelAPI;
import com.fs.starfarer.api.combat.EngagementResultAPI;
import com.fs.starfarer.api.ui.ValueDisplayMode;


public class AsteriodInteractionDialogPlugin implements InteractionDialogPlugin {

	// JANINO doesn't support enums, which would be the better choice here.
	// So, we'll use static data members instead.
	// Each represents an option the player can pick/interact with in the dialog,
	// except for "INIT", which is represents the dialog being opened.
	private static class OptionId {
		public static OptionId INIT = new OptionId();
		public static OptionId SURVEY = new OptionId();
		public static OptionId SUPPLY_PICKER = new OptionId();
		public static OptionId GET_SUPPLIES = new OptionId();
		public static OptionId LEAVE = new OptionId();
	}
	
	// Data we want to keep track of for each asteroid that was interacted with.
	public static class AsteroidData {
		public boolean investigated = false;
		public boolean hadSupplies = false;
		public int remainingSupplies = 0;
	}
	
	private InteractionDialogAPI dialog;
	private TextPanelAPI textPanel;
	private OptionPanelAPI options;
	private VisualPanelAPI visual;
	
	private CampaignFleetAPI playerFleet;
	private SectorEntityToken asteroid;
	private AsteroidData data;
	private SectorAPI sector;
	
	
	// Getting the color from settings.json
	private static final Color HIGHLIGHT_COLOR = Global.getSettings().getColor("buttonShortcut");
	
	public void init(InteractionDialogAPI dialog) {
		this.dialog = dialog;
		textPanel = dialog.getTextPanel();
		options = dialog.getOptionPanel();
		visual = dialog.getVisualPanel();

		sector = Global.getSector();
		playerFleet = sector.getPlayerFleet();
		asteroid = (SectorEntityToken) dialog.getInteractionTarget();
		data = getDataFor(asteroid);
		
		visual.setVisualFade(0.25f, 0.25f);
		
		if (asteroid.getCustomInteractionDialogImageVisual() != null) {
			visual.showImageVisual(asteroid.getCustomInteractionDialogImageVisual());
		} else {
			visual.showImagePortion("illustrations", "space_wreckage", 800, 800, 0, 0, 400, 400);
		}
	
		dialog.setOptionOnEscape("Leave", OptionId.LEAVE);
		
		optionSelected(null, OptionId.INIT);
	}
	
	public void backFromEngagement(EngagementResultAPI result) {
		// no combat here, so this won't get called
	}
	
	public void optionSelected(String text, Object optionData) {
		if (optionData == null) return;
		
		OptionId option = (OptionId) optionData;
		
		if (text != null) {
			textPanel.addParagraph(text, Global.getSettings().getColor("buttonText"));
		}

		if (option == OptionId.INIT) {
			addText("Your fleet approaches the asteroid.");
			if (data.investigated) {
				addText("You've already surveyed this asteroid. ");
				if (data.hadSupplies) {
					if (data.remainingSupplies > 0) {
						appendText("The supply cache you found has " + data.remainingSupplies + " crate(s) of supplies remaining.");
						textPanel.highlightFirstInLastPara("" + data.remainingSupplies, HIGHLIGHT_COLOR);
					} else {
						appendText("The supply cache you found earlier is all gone.");
					}
				} else {
					appendText("There is nothing of interest on the surface.");
				}
			}
			updateOptions();
		} else if (option == OptionId.SURVEY) {
			data.investigated = true;
			if (Math.random() > 0.5f) {
				int supplies = (int) (Math.random() * 100. + 1.);
				data.hadSupplies = true;
				data.remainingSupplies = supplies;
				addText("You send down a survey team and they report locating a hidden cache with " + data.remainingSupplies + " crate(s) of supplies!");
				textPanel.highlightFirstInLastPara("" + data.remainingSupplies, HIGHLIGHT_COLOR);
			} else {
				addText("You send down a survey team, but they report nothing of interest on the surface.");
			}
			updateOptions();
		} else if (option == OptionId.GET_SUPPLIES) {
			int quantity = (int) Math.round(options.getSelectorValue(OptionId.SUPPLY_PICKER));
			if (quantity > 0) {
				data.remainingSupplies -= quantity;
				addText("Your crew retrieves " + quantity + " crate(s) of supplies from the cache.");
				textPanel.highlightFirstInLastPara("" + quantity, HIGHLIGHT_COLOR);
				if (data.remainingSupplies > 0) {
					addText("After they're done, " + data.remainingSupplies + " crate(s) of supplies remain.");
					textPanel.highlightFirstInLastPara("" + data.remainingSupplies, HIGHLIGHT_COLOR);
				} else {
					addText("The supply cache is now empty.");
				}
				playerFleet.getCargo().addSupplies(quantity);
			}
			updateOptions();
		} else if (option == OptionId.LEAVE) {
			Global.getSector().setPaused(false);
			dialog.dismiss();			
		}

	}
	
	private void updateOptions() {
		options.clearOptions();
		
		if (data.remainingSupplies > 0) {
			options.addSelector("Select amount of supplies to take",
								OptionId.SUPPLY_PICKER, HIGHLIGHT_COLOR, 250, 35, 0, data.remainingSupplies,
								ValueDisplayMode.VALUE,
								"Use this control to select the number of supply crates to pick up.");
			options.addOption("Get the supplies", OptionId.GET_SUPPLIES, null);
		} else if (!data.investigated) {
			options.addOption("Send down a survey team", OptionId.SURVEY, "Investigate this asteroid for anything of interest");
		}
		
		options.addOption("Leave", OptionId.LEAVE, null);
	}

	
	private static final String SET_ID_IN_PERSISTENT_DATA = "AsteriodInteractionDialogPlugin_asteroidData";	
	// This method manages the AsteroidData we store in the persistent data store.
	// Will either return an existing AsteroidData, or create a new one,
	// put it in the data store, and return it.
	@SuppressWarnings("unchecked")
	private AsteroidData getDataFor(SectorEntityToken asteroid) {
		Map map = (HashMap) sector.getPersistentData().get(SET_ID_IN_PERSISTENT_DATA);
		if (map == null) {
			map = new HashMap();
			sector.getPersistentData().put(SET_ID_IN_PERSISTENT_DATA, map);			
		}
		
		AsteroidData data = (AsteroidData) map.get(asteroid);
		if (data == null) {
			data = new AsteroidData();
			map.put(asteroid, data);
		}
		return data;
	}

	private void addText(String text) {
		textPanel.addParagraph(text);
	}
	
	private void appendText(String text) {
		textPanel.appendToLastParagraph(" " + text);
	}
	
	private OptionId lastOptionMousedOver = null;
	public void optionMousedOver(String optionText, Object optionData) {
		lastOptionMousedOver = (OptionId) optionData;
	}
	
	public void advance(float amount) {
		
	}
	
	public Object getContext() {
		return null;
	}
}
